const express = require('express');

const enrollmentProjectionApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const config = require(`${libs}/config`);

let rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValueToField({
    name: 'state',
    table: 'estado',
    tableField: ['nome', 'id'],
    resultField: ['state_name', 'state_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'projecao_matricula'
    }
}, 'dims').addValueToField({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'projecao_matricula'
    }
}, 'filter').addValueToField({
    name: 'city',
    table: 'municipio',
    tableField: ['nome', 'id'],
    resultField: ['city_name', 'city_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'projecao_matricula'
    }
}, 'dims').addValueToField({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'projecao_matricula'
    }
}, 'filter').addValue({
    name: 'min_year',
    table: 'projecao_matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'projecao_matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
});

enrollmentProjectionApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field("'Brasil'", 'name')
    .field('SUM(projecao_matricula.urbano_dia_total)', 'urban_day_total')
    .field('SUM(projecao_matricula.urbano_noite_total)', 'urban_night_total')
    .field('SUM(projecao_matricula.rural_dia_total)', 'rural_day_total')
    .field('SUM(projecao_matricula.rural_noite_total)', 'rural_night_total')
    .field('projecao_matricula.etapa_ensino_escola_ano_id', 'education_level_school_year_id')
    .field('projecao_matricula.ano_censo', 'year')
    .from('projecao_matricula')
    .group('projecao_matricula.etapa_ensino_escola_ano_id')
    .group('projecao_matricula.ano_censo')
    .order('projecao_matricula.ano_censo')
    .order('projecao_matricula.etapa_ensino_escola_ano_id');

    next();
}, query, id2str.transform(), response('enrollment_projection'));

module.exports = enrollmentProjectionApp;